package main

import (
	"fmt"

	"git.coopgo.io/coopgo-platform/agenda/grpcapi"
	"git.coopgo.io/coopgo-platform/agenda/handlers"
	"git.coopgo.io/coopgo-platform/agenda/storage"
)

func main() {

	cfg, err := ReadConfig()
	if err != nil {
		panic(err)
	}

	var (
		service_name = cfg.GetString("name")
		grpc_enable  = cfg.GetBool("services.grpc.enable")
	)

	storage, err := storage.NewStorage(cfg)
	if err != nil {
		panic(err)
	}

	handler := handlers.NewHandler(cfg, storage)

	fmt.Println("Running", service_name, ":")

	failed := make(chan error)

	if grpc_enable {
		go grpcapi.Run(failed, cfg, handler)
	}

	// Terminate if one of the services fails

	err = <-failed

	fmt.Println("Terminating :", err)
}
